#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#

'''Collection of disparate utilities.

`xoutil` is essentially an extension to the Python's standard library, it does
not turn into a full framework, but it's very useful to be used from a
diversity of scenarios where compatibility is an important issue.

'''

# TODO: Deprecate this import
from .symbols import Unset, Undefined, Ignored, Invalid  # noqa
